@@ -11,12 +11,12 @@ import (
 
 // Peripheral abstraction layer for the MIMXRT1062
 
+const deviceName = nxp.Device
+
 func CPUFrequency() uint32 {
 	return 600000000
 }
 
-type PinMode uint8
-
 const (
 	// GPIO
 	PinInput PinMode = iota
