@@ -13,7 +13,7 @@ import (
 // Ever-incrementing pointer: no memory is freed.
 var heapptr = heapStart
 
-func alloc(size uintptr) unsafe.Pointer {
+func alloc(size uintptr, layout unsafe.Pointer) unsafe.Pointer {
 	// TODO: this can be optimized by not casting between pointers and ints so
 	// much. And by using platform-native data types (e.g. *uint8 for 8-bit
 	// systems).
@@ -35,6 +35,18 @@ func alloc(size uintptr) unsafe.Pointer {
 	return unsafe.Pointer(addr)
 }
 
+func realloc(ptr unsafe.Pointer, size uintptr) unsafe.Pointer {
+	newAlloc := alloc(size, nil)
+	if ptr == nil {
+		return newAlloc
+	}
+	// according to POSIX everything beyond the previous pointer's
+	// size will have indeterminate values so we can just copy garbage
+	memcpy(newAlloc, ptr, size)
+
+	return newAlloc
+}
+
 func free(ptr unsafe.Pointer) {
 	// Memory is never freed.
 }
@@ -52,7 +64,13 @@ func SetFinalizer(obj interface{}, finalizer interface{}) {
 }
 
 func initHeap() {
-	// Nothing to initialize.
+	// preinit() may have moved heapStart; reset heapptr
+	ptr := heapStart
+	if GOARCH == "wasm" {
+		// llvm11 and llvm12 do not correctly align the heap on wasm
+		ptr = align(ptr)
+	}
+	heapptr = ptr
 }
 
 // setHeapEnd sets a new (larger) heapEnd pointer.
