@@ -58,7 +58,7 @@ func main() {
 	initInternal()
 
 	run()
-	abort()
+	exit(0)
 }
 
 func initSystem() {
@@ -140,7 +140,7 @@ func initSystem() {
 
 	// now program the clock dividers
 	// config divisors: 180 MHz core, 60 MHz bus, 25.7 MHz flash, USB = IRC48M
-	nxp.SIM.CLKDIV1.Set((0 << nxp.SIM_CLKDIV1_OUTDIV1_Pos) | (2 << nxp.SIM_CLKDIV1_OUTDIV2_Pos) | (0 << nxp.SIM_CLKDIV1_OUTDIV1_Pos) | (6 << nxp.SIM_CLKDIV1_OUTDIV4_Pos))
+	nxp.SIM.CLKDIV1.Set((0 << nxp.SIM_CLKDIV1_OUTDIV1_Pos) | (2 << nxp.SIM_CLKDIV1_OUTDIV2_Pos) | (0 << nxp.SIM_CLKDIV1_OUTDIV3_Pos) | (6 << nxp.SIM_CLKDIV1_OUTDIV4_Pos))
 	nxp.SIM.CLKDIV2.Set((0 << nxp.SIM_CLKDIV2_USBDIV_Pos))
 
 	// switch to PLL as clock source, FLL input = 16 MHz / 512
@@ -226,10 +226,12 @@ func initInternal() {
 	// 	analog_init();
 }
 
-func postinit() {}
-
 func putchar(c byte) {
-	machine.PutcharUART(&machine.UART0, c)
+	machine.PutcharUART(machine.UART0, c)
+}
+
+func exit(code int) {
+	abort()
 }
 
 func abort() {
@@ -256,9 +258,9 @@ func abort() {
 		// keep polling some communication while in fault
 		// mode, so we don't completely die.
 		// machine.PollUSB(&machine.USB0)
-		machine.PollUART(&machine.UART0)
-		machine.PollUART(&machine.UART1)
-		machine.PollUART(&machine.UART2)
+		machine.PollUART(machine.UART0)
+		machine.PollUART(machine.UART1)
+		machine.PollUART(machine.UART2)
 	}
 }
 
